const fs = require('fs')
const data = require('./data.json')
const prettier = require('prettier')
const path = require('path')
const files = require('./files.json')
const { spawnSync } = require('child_process')
const remarkParse = require('remark-parse')
const remarkStringify = require('remark-stringify')
const unified = require('unified')
const youtube = require('./sitegin/utils/youtube')

const importUsers = true
const formatMD = true
const zip = true

const fileMap = new Map()
for (const entry of files) {
  fileMap.set(entry.source, entry.target)
}

/*
Manuální změny:
  - k článku 144 MHz PA s 2x GI7B přidat autora OK1GP (k Jirka, OK1UBO)
  - Inline obrázky v článcích
    webkamera
    2018/a1-contest
    2015/ii-subregional-2015
    2014/ii-subregional-2014
    2013/pajeni-pretavenim
    2013/vhf-contest-2013
    2011/balun-12
    2011/elektronicka-ruleta
    2011/webkamera-na-blatenskem-vrchu-opet-v-provozu
    2010/vystupni-dolni-propust-pro-pa-1kw-na-144-mhz
    2009/antenni-analyzer-vk5jst
    2009/terenni-profily-blatensky-vrch
*/

function mdFormat(md) {
  if (!formatMD) return md.replace(/\r\n/g, '\n')
  return prettier.format(md, {
    parser: 'markdown',
    endOfLine: 'lf',
    proseWrap: 'never',
  })
}
const example = {
  version: '0.3.1',
  atoms: [],
  cards: [
    [
      'image',
      {
        src: 'http://localhost:2368/content/images/2019/10/characters.jpg',
        caption: 'Caption',
        alt: 'Alt',
      },
    ],
    [
      'gallery',
      {
        images: [
          {
            fileName: 'characters.jpg',
            row: 0,
            width: 800,
            height: 492,
            src:
              'http://localhost:2368/content/images/2019/10/characters-1.jpg',
          },
          {
            fileName: 'rarity.jpg',
            row: 0,
            width: 1179,
            height: 1003,
            src: 'http://localhost:2368/content/images/2019/10/rarity.jpg',
          },
          {
            fileName: 'sq.png',
            row: 0,
            width: 6806,
            height: 6806,
            src: 'http://localhost:2368/content/images/2019/10/sq.png',
          },
        ],
      },
    ],
  ],
  markups: [['em'], ['strong']],
  sections: [
    [1, 'p', [[0, [], 0, 'Nadpis']]],
    [1, 'p', [[0, [], 0, 'Nadpis 2']]],
    [
      1,
      'p',
      [
        [0, [], 0, 'Text s '],
        [0, [0], 1, 'italic '],
        [0, [], 0, 'a '],
        [0, [1], 1, 'tučným'],
        [0, [], 0, ' písmem.'],
      ],
    ],
    [10, 0],
    [10, 1],
    [1, 'p', [[0, [], 0, 'normal text block']]],
  ],
}

function remapUrl(url_, page) {
  let url /* string */ = url_
  url = fileMap.get(url.startsWith('/') ? url.substring(1) : page.file + url)
  return url ? `/${url}` : url
}

const pipe = unified()
  .use(remarkParse)
  .use(remarkStringify)

function remapUrlMd(tree, page) {
  function change(ast) {
    if (ast.type === 'image') {
      const url = remapUrl(ast.url, page)
      if (url) {
        return { ...ast, url }
      }
      // console.warn('Not found:', page.file, ast.url)
    } else if (ast.type === 'imageReference') {
      console.log(page.file, ast)
    } else if (
      ![
        'text',
        'thematicBreak',
        'html',
        'code',
        'break',
        'inlineCode',
        'paragraph',
        'root',
        'link',
        'list',
        'listItem',
        'table',
        'tableRow',
        'tableCell',
        'strong',
        'emphasis',
        'blockquote',
        'linkReference',
        'heading',
      ].includes(ast.type)
    ) {
      console.log(ast.type)
    }
    return ast
  }
  function walk(ast) {
    const changed = change(ast)
    if (Array.isArray(changed.children)) {
      return { ...changed, children: changed.children.map(walk) }
    }
    return changed
  }
  return walk(tree)
}

function convertMd(md, page) {
  const result = {
    version: '0.3.1',
    markups: [],
    atoms: [],
    cards: [],
    sections: [],
  }
  function addCard(name, payload) {
    result.sections.push([10, result.cards.length])
    result.cards.push([name, payload])
  }
  function addMDCard(markdown) {
    addCard('markdown', { cardName: 'markdown', markdown: mdFormat(markdown) })
  }
  function addSimpleText(string) {
    result.sections.push([1, 'p', [[0, [], 0, string]]])
  }
  function createTree(children) {
    return {
      type: 'root',
      children: Array.isArray(children) ? children : [children],
    }
  }
  const tree = pipe.parse(md)
  const maxHead = tree.children
    .filter(p => p.type === 'heading')
    .map(p => p.depth)
    .reduce((a, b) => Math.min(a, b), 6)
  for (const part of tree.children) {
    if (part.type === 'paragraph') {
      if (part.children.length === 1) {
        const tag = part.children[0]

        if (tag.type === 'image') {
          const src = remapUrl(tag.url, page)
          if (src) {
            addCard('image', {
              src,
              alt: tag.alt || undefined,
              caption: tag.title || undefined,
            })
            continue
          }
        }
      }
      if (part.children.every(ch => ch.type === 'text')) {
        addSimpleText(part.children.map(ch => ch.value).join(''))
        continue
      }
      if (
        part.children.every(
          ch =>
            (ch.type === 'image' && ch.url.includes('youtu')) ||
            (ch.type === 'text' && ch.value.replace(/\./g, '').trim() === ''),
        )
      ) {
        for (const child of part.children) {
          if (child.type === 'image') {
            const info = youtube.info(child.url)
            addCard('embed', {
              url: child.url,
              html: `<iframe width="459" height="344" src="https://www.youtube.com/embed/${info.videoid}?modestbranding&start=${info.time}" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>`,
              type: 'video',
              caption: child.alt,
            })
          }
        }
        continue
      }
      if (
        part.children.every(
          ch =>
            (ch.type === 'image' && remapUrl(ch.url, page)) ||
            (ch.type === 'text' && ch.value.replace(/\./g, '').trim() === ''),
        )
      ) {
        // console.log('gallery', page.file)
        addCard('gallery', {
          images: part.children
            .filter(ch => ch.type === 'image')
            .map((ch, i) => ({
              fileName: ch.url,
              row: Math.floor(i / 3),
              src: remapUrl(ch.url, page),
            })),
        })
        continue
      }
    } else if (part.type === 'heading') {
      if (part.children.every(c => c.type === 'text')) {
        const level = part.depth === maxHead ? 'h2' : 'h3'
        const text = part.children.map(c => c.value).join('')
        result.sections.push([1, level, [[0, [], 0, text]]])
        continue
      }
    }
    // falback
    addMDCard(pipe.stringify(remapUrlMd(createTree(part), page)))
  }

  return result
}

function truncate(text, maxLength = 255) {
  if (text.length < maxLength) return text
  const parts = text.split(' ')
  let res = ''
  for (let i = 0; i < parts.length; i++) {
    if (res.length + parts[i].length + 4 <= maxLength) res += ` ${parts[i]}`
    else break
  }
  return `${res}...`
}

;(async () => {
  console.log(new Date().toISOString(), 'start')
  const users = [
    { name: 'Martin Vítek', email: 'fotomartinkv@gmail.com' },
    { name: 'Vláďa, OK1FIK', email: 'ok1fik@atlas.cz', slug: 'ok1fik' },
    {
      name: 'Isabella Skořepová (import)',
      email: 'isabella@ok1kvk.cz',
      slug: 'codewitchbella',
    },
    { name: 'Jirka, OK1UBO', email: 'midgebill@seznam.cz', slug: 'ok1ubo' },
    { name: 'Michal, OK1WMR', email: 'ok1wmr@gmail.com', slug: 'ok1wmr' },
    { name: 'Zdeněk, OK1ZES', email: 'zdenek.svub@centrum.cz', slug: 'ok1zes' },
    { name: 'Míra, OK7KM', email: 'miroslav.koncik@gmail.com', slug: 'ok7km' },
    { name: 'Administrator', email: 'administrator@ok1kvk.cz' },
    { name: 'Vašek Král', email: 'vendakral@seznam.cz' },
    { name: 'Petr, OK1WPN', email: 'ok1wpn@ok1kvk.cz', slug: 'ok1wpn' },
    { name: 'OK1GP', email: 'ok1gp@ok1kvk.cz', slug: 'ok1gp' },
  ].map((u, i) => ({ ...u, id: i + 2 }))
  const userRemap = {
    'OK1GP, OK1UBO': 'Jirka, OK1UBO',
    'Michal Rybka': 'Michal, OK1WMR',
    'Mira, OK7KM': 'Míra, OK7KM',
    'Jakub Skořepa': 'Isabella Skořepová (import)',
    'Isabella Skořepová': 'Isabella Skořepová (import)',
  }
  const userByName = new Map()
  for (const u of users) userByName.set(u.name, u.id)
  for (const [from, to] of Object.entries(userRemap))
    userByName.set(from, userByName.get(to))

  const tagMap = new Map()
  const tags = data.tags
    .map(tag => ({
      name: tag.tag.text,
      slug: tag.tag.url.replace(/^tag\//, ''),
      query: tag.tag.url,
    }))
    .concat([{ name: '#nodate', query: '#nodate' }])
    .map((tag, i) => {
      const res = {
        name: tag.name,
        slug:
          tag.slug === 'c/c++'
            ? 'c-cpp'
            : tag.slug === 'rozbite-clanky'
            ? '#rozbite-clanky'
            : tag.slug,
        id: i + 1,
        description: '',
      }
      tagMap.set(tag.query, res.id)
      return res
    })

  console.log(new Date().toISOString(), 'posts')
  const postsTags = []
  const posts = await Promise.all(
    data.pages.map(async (page, i) => {
      const id = i + 1

      // .processSync(page.content).contents

      // console.log(stringified)

      const mobiledoc = JSON.stringify(convertMd(page.content, page))
      for (const tag of page.metadata.tags) {
        postsTags.push({ tag_id: tagMap.get(tag.url), post_id: id })
      }
      if (page.metadata.nodate)
        postsTags.push({ tag_id: tagMap.get('#article'), post_id: id })
      return {
        id,
        title: page.metadata.title,
        slug: page.file.replace(/^clanek\//, '').replace(/\/$/, ''),
        mobiledoc,
        page: page.metadata.notarticle || false,
        status: 'published',
        published_at: Date.parse(page.metadata.date.creation),
        published_by: importUsers
          ? userByName.get(page.metadata.author.name)
          : undefined,
        author_id: importUsers
          ? userByName.get(page.metadata.author.name)
          : undefined,
        created_at: Date.parse(page.metadata.date.creation),
        updated_at: Date.parse(page.metadata.date.modification),
        custom_excerpt: page.metadata.perex
          ? truncate(page.metadata.perex, 300)
          : page.metadata.perex_e
          ? truncate(page.metadata.perex_e, 300)
          : undefined,
        feature_image: remapUrl(page.metadata.image || '', page),
      }
    }),
  )
  console.log(new Date().toISOString(), 'write')
  const ghost = {
    meta: {
      exported_on: Date.now(),
      version: '2.14.0',
    },
    data: {
      posts,
      tags,
      users: importUsers ? users : [],
      posts_tags: postsTags,
      posts_authors: [],
      roles_users: [],
    },
  }

  fs.writeFileSync('import/ghost.json', JSON.stringify(ghost, null, 2), 'utf8')
  console.log(new Date().toISOString(), 'zipping')
  if (zip)
    spawnSync('zip', ['-r', 'ghost.zip', 'content', 'ghost.json'], {
      cwd: path.join(__dirname, 'import'),
    })

  console.log(new Date().toISOString(), 'done')
})().catch(e => console.error(e))
